use std::path::{PathBuf, Path};
use std::time::SystemTime;

use chrono::{DateTime, Local};

use crate::event::Event;



///
pub struct MonitoredFiles
{
    ///
    mod_dates: Vec<SystemTime>,

    ///
    paths: Vec<PathBuf>
}



impl MonitoredFiles
{
    pub fn new() -> Self
    {
        MonitoredFiles
        {
            mod_dates: Vec::new(),
            paths: Vec::new()
        }
    }

    pub fn process_event(&mut self, path: &Path, event: Event)
    {
        match event
        {
            Event::New(creation_time) =>
            {
                self.add(path, creation_time);
            }

            Event::Modify(mod_time) =>
            {
                self.modify(&path, mod_time);
            }

            Event::Delete =>
            {
                self.remove(&path);
            }
        }
    }

    pub fn add(&mut self, path: &Path, time: SystemTime)
    {
        self.paths.push(PathBuf::from(path));
        self.mod_dates.push(time);
    }

    pub fn modify(&mut self, path: &Path, time: SystemTime)
    {
        if let Some(index) = self.get_path_index(path)
        {
            self.mod_dates[index] = time;
        }
    }

    pub fn remove(&mut self, path: &Path)
    {
        if let Some(index) = self.get_path_index(path)
        {
            self.mod_dates.remove(index);
            self.paths.remove(index);
        }
    }

    pub fn print(&self, base: &Path)
    {
        for file in 0..self.paths.len()
        {
            let date_time: DateTime<Local> = DateTime::from(self.mod_dates[file]);

            match &self.paths[file].strip_prefix(base)
            {
                Ok(rel_path) =>
                {
                    println!("[{}] {}", date_time.format("%d/%m/%Y %H:%M"), rel_path.display());
                }

                Err(e) =>
                {
                    eprintln!("Unable to strip base directory:  {}", e);
                }
            }
        }
    }

    fn get_path_index(&self, path: &Path) -> Option<usize>
    {
        for (index, stored_path) in self.paths.iter().enumerate()
        {
            if path == stored_path
            {
                return Some(index);
            }
        }

        None
    }
}
